@@ -225,6 +225,33 @@ def fetch_crypto_minutely_data(ssj=None, local_root_directory=None, minutely_ret
         dataframe = pd.read_pickle(pickle_saving_path)
     return dataframe
 
+def fetch_local_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_pkl_filename_suffix='_hourly_returns.pkl',refetch_all=True, daily_crypto_starting_day='2012-01-01', daily_crypto_ending_day=None, ssj_against='USDT', exchange = 'binance',local_path= f'/Users/stefanduprey/Documents/My_StephaneStrategy/'):
+    dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
+
+    pickle_saving_path = local_root_directory + ssj + '_' + dates_stub + hourly_return_pkl_filename_suffix
+
+    if ssj == 'BTC':
+        dataframe = pd.read_pickle(local_path + f'btc_hourly.pkl')
+
+    elif ssj == 'ETH':
+        dataframe = pd.read_pickle(local_path + f'eth_hourly.pkl')
+
+    dataframe.columns =  ['date','close']
+
+    dataframe['date'] = pd.to_datetime(dataframe['date'])
+    dataframe = dataframe.set_index(dataframe['date'])
+    dataframe = dataframe.drop(columns=['date'])
+    dataframe = dataframe.sort_index()
+    dataframe['open'] = dataframe['close'].shift(1)
+
+    print('size fetched')
+    print(dataframe.shape)
+    dataframe = dataframe[dataframe.index >= daily_crypto_starting_day]
+    print('size filtered after '+str(daily_crypto_starting_day))
+    print(dataframe.shape)
+    dataframe.to_pickle(pickle_saving_path)
+    return dataframe
+
 
 def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_pkl_filename_suffix='_hourly_returns.pkl',refetch_all=True, daily_crypto_starting_day='2012-01-01', daily_crypto_ending_day=None, ssj_against='USDT', exchange = 'binance'):
     dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
