@@ -38,7 +38,7 @@ def buildAdvancedPriceFeatures(quotes_df=None, lookback_window=20, rolling_direc
 
 
 class AbstractAssembler(ABC):
-    def __init__(self, starting_date = None, running_date=None, drop_token=None, dropbox_backup=True, returns_pkl_file_suffix='_returns.pkl', features_pkl_file_suffix='_features.pkl', ts_features_pkl_file_suffix='tsfresh_features.pkl', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy',shortcut_features_saving_suffix='_shortcut_features.npy', shortcut_features_names_saving_suffix='_shortcut_features_names.npy',shortcut_features_pkl_file_name = 'shortcut.pkl', local_root_directory='../data/', user='napoleon',macro_features_directory = 'features_macro', rolling_direction=1):
+    def __init__(self, starting_date = None, running_date=None, drop_token=None, dropbox_backup=True, returns_pkl_file_suffix='_returns.pkl', features_pkl_file_suffix='_features.pkl', ts_features_pkl_file_suffix='tsfresh_features.pkl', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy',shortcut_features_saving_suffix='_shortcut_features.npy', shortcut_features_names_saving_suffix='_shortcut_features_names.npy',shortcut_features_pkl_file_name = 'shortcut.pkl', local_root_directory='../data/', user='napoleon',macro_features_directory = 'features_macro', rolling_direction=1, crypto_ticker = None):
         super().__init__()
         self.local_root_directory = local_root_directory
         self.user = user
@@ -57,7 +57,7 @@ class AbstractAssembler(ABC):
         self.rolling_direction = rolling_direction
 
         self.features_pkl_file_name = running_date.strftime('%d_%b_%Y')+features_pkl_file_suffix
-        self.returns_pkl_file_name = running_date.strftime('%d_%b_%Y')+returns_pkl_file_suffix
+
         self.ts_features_pkl_file_name = running_date.strftime('%d_%b_%Y')+ts_features_pkl_file_suffix
 
         self.shortcut_features_saving_suffix = shortcut_features_saving_suffix
@@ -70,6 +70,12 @@ class AbstractAssembler(ABC):
         self.ret_feat_df_suffix = '_ret_feat_df.pkl'
         self.truncating_features = False
 
+        self.crypto_ticker = crypto_ticker
+        if self.crypto_ticker is not None:
+            self.returns_pkl_file_name =  self.crypto_ticker + '_' +self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
+        else :
+            self.returns_pkl_file_name = self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
+
     @abstractmethod
     def assembleFeature(self, feature_type, n_past_features, tsfresh_lookback_window):
         pass
