@@ -27,7 +27,6 @@ def create_quote_signal_figure(data_df, signals_df, strategy_name):
     return fig
 
 
-
 def plot_multiple_time_series(data_df):
     fig = make_subplots(rows=1, cols=1)
     constituents = [col for col in data_df.columns if col not in ['date','Date']]
@@ -43,6 +42,42 @@ def plot_multiple_time_series(data_df):
     fig.update_xaxes(rangeslider_visible=True)
     return fig
 
+def plot_multiple_time_series(data_df, split =False, logy = True):
+    if split:
+        fig = make_subplots(rows=1, cols=len(data_df.columns))
+        constituents = [col for col in data_df.columns if col not in ['date','Date']]
+        counter = 1
+        for me_constituent in constituents:
+            trace_sig = go.Scatter(
+                x=data_df.index,
+                y=data_df[me_constituent],
+                name=me_constituent,
+                opacity=0.8)
+            fig.append_trace(trace_sig, row=1, col=counter)
+            if logy:
+                fig.update_yaxes(title_text="y-axis in logarithmic scale", type="log", row=1, col=counter)
+            counter = counter+1
+
+        fig.update_layout(height=600, width=1600, title_text="Strategy performances",showlegend=True)
+        fig.update_xaxes(rangeslider_visible=True)
+        return fig
+    else:
+        fig = make_subplots(rows=1, cols=1)
+        constituents = [col for col in data_df.columns if col not in ['date','Date']]
+        for me_constituent in constituents:
+            trace_sig = go.Scatter(
+                x=data_df.index,
+                y=data_df[me_constituent],
+                name=me_constituent,
+                opacity=0.8)
+            fig.append_trace(trace_sig, row=1, col=1)
+
+        if logy:
+            fig.update_yaxes(title_text="y-axis in logarithmic scale", type="log", row=1, col=1)
+        fig.update_layout(height=600, width=1600, title_text="Strategy performances",showlegend=True)
+        fig.update_xaxes(rangeslider_visible=True)
+        return fig
+
 def plot_multiple_turnover(data_df, ma = None):
     fig = make_subplots(rows=1, cols=1)
     constituents = [col for col in data_df.columns if col not in ['date','Date']]
