@@ -72,6 +72,33 @@ def generate_volume_filter_rsi(lags, middles, low_levs, high_levs, contravariant
                             parameters.append((lag, middle, low_lev, high_lev, contravariant,colmun))
     return parameters
 
+rolling_windows = [5,10,20,50,100,150,200,250,300,350,400,450,500]
+
+instantaneous_slope_fin_diff_hs = [1,2,5,10,15]
+thresholds = [0,0.01,0.05,0.1]
+signal_vol_windows = [1,3,5,10,15,20]
+
+def generate_fin_diffs(instantaneous_slope_fin_diff_hs, rolling_windows, thresholds, signal_vol_windows):
+    parameters = []
+    for fin_h in instantaneous_slope_fin_diff_hs:
+        for rolling_window in rolling_windows:
+            for threshold in thresholds:
+                for signal_vol_window in signal_vol_windows:
+                    parameters.append((fin_h,rolling_window,threshold,signal_vol_window))
+    return parameters
+
+def generate_fin_diffs_advanced(instantaneous_slope_fin_diff_hs, rolling_windows, thresholds, signal_vol_windows, slope_windows, second_rolling_windows):
+    parameters = []
+    for fin_h in instantaneous_slope_fin_diff_hs:
+        for rolling_window in rolling_windows:
+            for threshold in thresholds:
+                for signal_vol_window in signal_vol_windows:
+                    for slope_window in slope_windows:
+                        for second_rolling_window in second_rolling_windows:
+                            parameters.append((fin_h,rolling_window,threshold,signal_vol_window, slope_window, second_rolling_window))
+    return parameters
+
+
 def generate_volume_filter_lead_lag(leads, lags, low_levs, high_levs, percentage_longs, coliumns):
     parameters = []
     for lead in leads:
