@@ -14,6 +14,7 @@ from napoleontoolbox.file_saver import dropbox_file_saver
 import time
 
 from napoleontoolbox.bot import binance_bot
+from datetime import timedelta
 
 # CryptoCompare
 
@@ -102,17 +103,42 @@ def binance_get_histo_future_balance_snapshot(api, p, request_date):
 
 def binance_get_future_balance_history(api, p, start_time, end_time):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
-    return bot.get_futures_balance_history(start_time, end_time)
+    temp_time = start_time
+    valos = None
+    while temp_time < end_time:
+        temp_time = min(end_time, start_time+timedelta(int(30)))
+        print(f'requesting between {start_time} and {temp_time}')
+        temp_valos = bot.get_futures_balance_history(start_time, temp_time)
+        start_time = temp_time
+        if valos is None:
+            valos = temp_valos.copy()
+        else:
+            valos = pd.concat([temp_valos, valos])
+
+    valos['date'] = pd.to_datetime(valos['date'])
+    valos = valos.sort_values(by=['date'])
+    return valos
+
+    # deltadays = end_time - start_time
+    # middletime = start_time+ timedelta(int(deltadays.days/2))
+    # first_valos = bot.get_futures_balance_history(start_time, middletime)
+    # second_valos = bot.get_futures_balance_history(middletime, end_time)
+    # return pd.concat([first_valos,second_valos])
 
 
 def binance_get_futures_transfer_histo(api, p, start_time, end_time):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
     return bot.get_futures_transfer_history(['USDT', 'BNB'], start_time, end_time)
 
-def binance_get_position_history(api, p, start_time, end_time):
+def binance_get_position_risk(api, p, start_time, end_time):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
-    return bot.get_futures_positions_margin_change_history(['ETH','BTC'], start_time, end_time)
+    positions_df = pd.DataFrame(bot.get_future_position_risk())
+    positions_df = positions_df[abs(positions_df['positionAmt'].astype(float)) > 0].copy()
+    return positions_df
 
+def binance_get_position_margin_history(api, p, start_time, end_time):
+    bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
+    return bot.get_futures_positions_margin_change_history(['BTCUSDT','ETHUSDT'], start_time, end_time)
 
 
 #https://binance-docs.github.io/apidocs/spot/en/#wallet-endpoints
@@ -129,8 +155,8 @@ def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggre
   client = Client(api, p)
   assert type_ in ['trade_history', 'futures_funding', 'futures_klines', 'wallet_history', 'spot_klines'], 'Wrong type_: trade_history, futures_funding, futures_klines, wallet_history available'
   if aggreg is not None:
-    assert aggreg in ['1h', '1m'], 'Wrong aggreg_: 1m, 1h available'
-    dct_cvt = {'1m': Client.KLINE_INTERVAL_1MINUTE, '1h':Client.KLINE_INTERVAL_1HOUR}
+    assert aggreg in ['1h', '1m', '1d'], 'Wrong aggreg_: 1m, 1h available'
+    dct_cvt = {'1m': Client.KLINE_INTERVAL_1MINUTE, '1h':Client.KLINE_INTERVAL_1HOUR, '1d':Client.KLINE_INTERVAL_1DAY}
     aggreg = dct_cvt[aggreg]
 
 
@@ -375,7 +401,13 @@ def wh_to_arr(df):
   df = df.replace('ETHUSDT', -1).replace('BTCUSDT',1).replace('FUNDING_FEE', 0).replace('COMMISSION', 1).replace('TRANSFER',2).replace('REALIZED_PNL', 3).replace('BNB',1).replace('USDT', 0)
   df = df[['symbol', 'incomeType', 'income', 'asset', 'time']].copy()
   #df = df.replace({'(.*?)':0}, regex=True)
-  df =df[df['symbol'] != '']
+  try:
+      filtered_df = df[df['symbol'] != '']
+      df = filtered_df.copy()
+  except:
+      symbols = df['symbol'].unique()
+      print(f'Unexpected error filtering {symbols}')
+
 
   df = df.astype(float)
   d = typed.Dict()
@@ -430,6 +462,247 @@ def numba_compute_fees(arr, d):
   arr, d = _merge_update(arr, d, arrays, names)
   return arr, d
 
+def fetch_binance_ohlc(symbol = 'BNBUSDT', frequence = '1d', public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', trade_only_granularity = False, recompute_all=False):
+    print('fetching daily tezos')
+    if recompute_all:
+        binance_df = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time,
+                                           symbol=symbol, aggreg=frequence)
+
+        binance_df.index = pd.to_datetime(binance_df['ts'])
+        binance_df['open'] = binance_df['open'].astype(float)
+        binance_df['close'] = binance_df['close'].astype(float)
+        binance_df['high'] = binance_df['high'].astype(float)
+        binance_df['low'] = binance_df['low'].astype(float)
+        save_equity_daily_data(data_df = binance_df, ticker = symbol, starting_date=start_time, running_date = end_time, local_root_directory = local_root_directory)
+    else :
+        filename = symbol + '_' + start_time.strftime('%d_%b_%Y') + '_' + end_time.strftime(
+            '%d_%b_%Y') + '_daily_returns.pkl'
+        binance_df = pd.read_pickle(local_root_directory + filename)
+
+    return binance_df
+
+def save_equity_daily_data(data_df = None, ticker=None, provider = 'napoleon', starting_date=None,running_date=None,local_root_directory=None):
+    filename = ticker + '_' + starting_date.strftime('%d_%b_%Y') + '_' + running_date.strftime(
+        '%d_%b_%Y') + '_daily_returns.pkl'
+    data_df.to_pickle(local_root_directory + filename)
+
+
+def fetch_all_necessary_data_b40lux(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', recompute_all=True, by_pass_month=False):
+    print('inside')
+    dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Binance'
+    #filename_stub = f'{inv_year_dictionary[year]}_{inv_month_dictionary[month]}_{inv_market_dictionary[market]}'
+
+    report_pkl_file_name = f'{filename_stub}_report_b40.pkl'
+
+    trade_pkl_file_name = f'{filename_stub}_trades_b40.pkl'
+
+    walletHist_pkl_file_name = f'{filename_stub}_walletHist_b40.pkl'
+
+    positions_pkl_file_name = f'{filename_stub}_positions_b40.pkl'
+
+    transfers_pkl_file_name = f'{filename_stub}_transfers_b40.pkl'
+
+    ohlc_bnb_min_binance_pkl_file_name = f'{filename_stub}_ohlc_bnb_min_binance.pkl'
+
+    if recompute_all:
+        fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
+
+        actualPositions = binance_get_position_risk(public_key,private_key,start_time, end_time)
+        history_balance = binance_get_future_balance_history(public_key,private_key,start_time, end_time)
+
+        trades = binance_get(public_key, private_key, 'trade_history', start_time, end_time=end_time, symbol ='ETHUSDT').drop_duplicates()
+        walletHist = binance_get(public_key, private_key, 'wallet_history', start_time, end_time= end_time, symbol ='ETHUSDT')
+
+        walletHist = walletHist[~walletHist['incomeType'].isin(['API_REBATE'])]
+
+
+        ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
+
+
+
+        history_balance = pd.merge(history_balance, ohlc_bnb_min_binance[['ts','open']], how='left', left_on = 'date', right_on = 'ts')
+
+
+        def convert_bnb_to_usdt(row):
+            print(row)
+            if row['asset'] == 'BNB':
+                return float(row['walletBalance']) * float(row['open'])
+            elif row['asset'] == 'USDT':
+                return float(row['walletBalance'])
+            else :
+                return np.nan
+
+
+        history_balance['convertedWalletBalance'] = history_balance.apply(convert_bnb_to_usdt, axis=1)
+
+        converted_history_balance = history_balance[['ts','convertedWalletBalance']].groupby('ts').sum()
+
+        if save_to_dropbox :
+            full_path = local_root_directory + trade_pkl_file_name
+            trades.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=trade_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + walletHist_pkl_file_name
+            walletHist.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=walletHist_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_bnb_min_binance_pkl_file_name
+            ohlc_bnb_min_binance.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_bnb_min_binance_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + report_pkl_file_name
+            converted_history_balance.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=report_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + positions_pkl_file_name
+            actualPositions.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=positions_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            if fut_transfers is not None:
+                full_path = local_root_directory + transfers_pkl_file_name
+                fut_transfers.to_pickle(full_path)
+                print(f'uploading to dropbox {full_path}')
+                dbx.upload(fullname=full_path, folder='', subfolder='', name=transfers_pkl_file_name, overwrite=True)
+                #            os.remove(full_path)
+
+
+    else:
+        # trades = dbx.download_pkl(trade_pkl_file_name)
+        # walletHist = dbx.download_pkl(walletHist_pkl_file_name)
+        # ohlc_bnb_min_binance = dbx.download_pkl(ohlc_bnb_min_binance_pkl_file_name)
+        # converted_history_balance = dbx.download_pkl(report_pkl_file_name)
+        # actualPositions = dbx.download_pkl(positions_pkl_file_name)
+
+        trades = pd.read_pickle(local_root_directory+trade_pkl_file_name)
+        walletHist = pd.read_pickle(local_root_directory+walletHist_pkl_file_name)
+        ohlc_bnb_min_binance = pd.read_pickle(local_root_directory+ohlc_bnb_min_binance_pkl_file_name)
+        converted_history_balance = pd.read_pickle(local_root_directory+report_pkl_file_name)
+        actualPositions = pd.read_pickle(local_root_directory+positions_pkl_file_name)
+#        fut_transfers = pd.read_pickle(local_root_directory + transfers_pkl_file_name)
+
+    # ohlc_bnb_binance (minutes)
+    ohlc_bnb_min_binance = ohlc_bnb_min_binance.set_index('ts')
+    assert isinstance(ohlc_bnb_min_binance.index, pd.DatetimeIndex)
+    ohlc_bnb_min_binance.columns = [x + '_bnb_bnc' for x in ohlc_bnb_min_binance.columns]
+    ohlc_bnb_min_binance.index = ohlc_bnb_min_binance.index.tz_localize(None)
+
+
+    ##################################
+    #################
+    ################# Positions
+    #################
+    ##################################
+    print(actualPositions.shape)
+    pos_columns = [
+        'symbol',
+        'positionAmt',
+        'markPrice'
+    ]
+    actualPositions = actualPositions[pos_columns].copy()
+    actualPositions = actualPositions.rename(columns={
+        'symbol':'asset',
+        'positionAmt':'amount',
+        'markPrice':'price'
+    })
+
+    ##################################
+    #################
+    ################# Trades
+    #################
+    ##################################
+
+    trades_df = trades.rename(columns={
+        'symbol': 'Security Code',
+        'commissionAsset': 'Currency',
+        'side': 'Oper',
+        'realizedPnl':'PnL',
+        'qty': 'Amount',
+        'price': 'Price',
+        'commission': 'Fees',
+        'ts': 'Date'
+    })
+
+    trades_df['Nature'] = 'Trade'
+    trades_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Amount', 'Price', 'PnL', 'Fees']
+    trades_df = trades_df[trades_columns]
+    futures_income_history_df = walletHist[
+        walletHist['incomeType'] == 'FUNDING_FEE'].copy()
+    futures_income_history_df['Nature'] = 'Funding'
+    futures_income_history_df = futures_income_history_df.rename(columns={
+        'symbol': 'Security Code',
+        'asset': 'Currency',
+        'incomeType': 'Oper',
+        'income': 'Fees',
+        'ts': 'Date'
+    })
+    futures_income_history_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Fees']
+    futures_income_history_df = futures_income_history_df[
+        futures_income_history_columns]
+    binance_df = pd.concat([trades_df, futures_income_history_df])
+    binance_df['Date'] = pd.to_datetime(binance_df['Date'])
+    binance_df = binance_df.sort_values(by=['Date'])
+
+    binance_df['Fees'] = -binance_df['Fees'].astype(float)
+    filteredWallet = converted_history_balance.copy()
+    converted_history_balance['Date'] = converted_history_balance.index
+    filteredWallet['Date'] = filteredWallet.index
+    filteredWallet['month'] = filteredWallet['Date'].dt.month
+    filteredWallet['insideMonth'] = filteredWallet['month'] == me_month
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['insideMonth'].shift(-1)
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['shiftedInsideMonth'].fillna(True)
+    filteredWallet = filteredWallet[filteredWallet['shiftedInsideMonth']].copy()
+    filteredWallet = filteredWallet.drop(columns=['month', 'insideMonth', 'shiftedInsideMonth'])
+
+    filteredWallet = filteredWallet.iloc[[0, -1]]
+
+    if not by_pass_month:
+        minDate = min(filteredWallet['Date'])
+        maxDate = max(filteredWallet['Date'])
+    else :
+        minDate = min(converted_history_balance.index)
+        maxDate = max(converted_history_balance.index)
+
+    filtered_binance_df = binance_df[binance_df['Date'] >= minDate].copy()
+    filtered_binance_df = filtered_binance_df[filtered_binance_df['Date'] <= maxDate].copy()
+
+    filteredWallet = filteredWallet.rename(columns = {
+        'convertedWalletBalance': 'wallet_balance_usdt'
+    })
+
+    converted_history_balance = converted_history_balance.rename(columns = {
+        'convertedWalletBalance': 'wallet_balance_usdt'
+    })
+
+    valo_columns=[
+        'Date',
+        'wallet_balance_usdt',
+    ]
+    b40_cols = ['Nature', 'Security Code', 'Currency', 'Description', 'Comment', 'Oper', 'Date', 'Value', 'Settlement',
+                'Amount', 'Price', 'Int. Rate', 'Meth', 'Excl', 'Last Coupon', 'Interest', 'Depo', 'Bank', 'Pmt Cur',
+                'Source	Investor', 'Fees', 'Bank', 'PnL']
+    for me_col in b40_cols:
+        if me_col not in filtered_binance_df.columns:
+            filtered_binance_df[me_col] = np.nan
+
+    filtered_binance_df = filtered_binance_df[b40_cols].copy()
+    return filtered_binance_df, filteredWallet[valo_columns], converted_history_balance[valo_columns], actualPositions
+
+
+
 
 
 def fetch_all_necessary_data(public_key = None, private_key= None, start_time= None, end_time= None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', trade_only_granularity = False):
@@ -455,6 +728,8 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
 
     ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
 
+    history_balance = binance_get_future_balance_history(public_key,private_key,start_time, end_time)
+
     fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
     if fut_transfers is not None:
         fut_transfers['date'] = fut_transfers['date'].dt.floor('H')
@@ -486,7 +761,6 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
 
 
 
-    history_balance = binance_get_future_balance_history(public_key,private_key,start_time, end_time)
 
     history_balance = pd.merge(history_balance, ohlc_bnb_min_binance[['ts','open']], how='left', left_on = 'date', right_on = 'ts')
 
