@@ -27,8 +27,9 @@ import warnings
 warnings.simplefilter("ignore")
 
 import os
+import datetime as dt
 
-
+from datetime import timedelta
 from napoleontoolbox.file_saver import dropbox_file_saver
 
 # Utils
@@ -262,7 +263,7 @@ class BitMEX:
                 url = gen_url_BitMEX(optype, currency, startdate, enddate, precision, 500)
                 output = pd.concat([output, self.retrieve_and_assess(s, url)], ignore_index=True)
                 enddate = str(output['timestamp'].iloc[-1])
-                time.sleep(3)
+                time.sleep(20)
 
             if rest != 0:
                 url = gen_url_BitMEX(optype, currency, startdate, enddate, precision)
@@ -360,9 +361,9 @@ def find_lastPx(arr, d):
 
 @jit(nopython=True)
 def get_qtraded(arr, d):
-    qtraded_ETH = [187465]  # Faut aller voir dans la feuille de trades sinon c nop
-    qtraded_BTC = [620484]
-    for i in range(1, arr.shape[0]):
+    qtraded_ETH = []  # Faut aller voir dans la feuille de trades sinon c nop
+    qtraded_BTC = []
+    for i in range(0, arr.shape[0]):
         if (arr[i, d['symbol']] == 1) and (arr[i, d['execType']] == 1):
             qtraded_ETH.append(0)
             qtraded_BTC.append(arr[i, d['lastQty']])
@@ -521,6 +522,357 @@ def get_fees(arr, d):
             fold_a.append(np.array(_))
         return fold_a
 
+def preprocess_bitmex_trades(trades):
+    df, d = trades_to_arr(trades)
+    df, d = numba_compute(df, d)
+    print(f'Done for {trades.shape[0]} trades')
+
+    df = pd.DataFrame(df, columns=[i for i in d.keys()])
+    df['funding_fees_xbt_usd'] = df.funding_fees_xbt_btc * df.lastPxBTC
+    df['funding_fees_eth_usd'] = df.funding_fees_eth_btc * df.lastPxBTC
+    df['trading_fees_eth_usd'] = df.trading_fees_eth_btc * df.lastPxBTC
+    df['trading_fees_xbt_usd'] = df.trading_fees_xbt_btc * df.lastPxBTC
+    df.transactTime = df.transactTime.apply(lambda x: dt.datetime.utcfromtimestamp(x / 1000))
+    df['total_fees_usd'] = df.trading_fees_xbt_usd + df.trading_fees_eth_usd + df.funding_fees_xbt_usd + df.funding_fees_eth_usd
+    df['quantity'] = df['traded_eth'] + df['traded_xbt']
+
+    df = df.replace('Buy', 1).replace('Sell', -1).replace('ETHUSD', -1).replace('XBTUSD', 1).replace('Limit',
+                                                                                                     0).replace(
+        'Market', 1).replace('Trade', 1).replace('Funding', 0)
+
+    df['symbol'] = df['symbol'].replace(-1., 'ETHUSD').replace(1., 'XBTUSD')
+    df['side'] = df['side'].replace(-1., 'Sell').replace(1., 'Buy')
+    df['execType'] = df['execType'].replace(0., 'Funding').replace(1., 'Trade')
+
+    return df
+    # td.index = td.index.tz_localize(None)
+    #
+    # print('Grouping by...')
+    # td = df[['dt_floor', 'traded_eth', 'traded_xbt', 'size_eth', 'size_xbt', 'sens_trade_eth', 'sens_trade_xbt',
+    #          'funding_fees_xbt_usd', 'funding_fees_eth_usd',
+    #          'trading_fees_xbt_btc', 'trading_fees_xbt_usd', 'trading_fees_eth_btc', 'trading_fees_eth_usd',
+    #          'total_fees_usd']].groupby('dt_floor').sum()
+
+
+
+
+def fetch_all_necessary_data_b40(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', recompute_all = True, by_pass_month = False):
+
+    dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Bitmex'
+    #filename_stub = f'{inv_year_dictionary[year]}_{inv_month_dictionary[month]}_{inv_market_dictionary[market]}'
+
+
+    reportvalo_pkl_file_name = f'{filename_stub}_report_valo_b40.pkl'
+    trade_pkl_file_name = f'{filename_stub}_trades_b40.pkl'
+    positions_pkl_file_name = f'{filename_stub}_positions_b40.pkl'
+
+    ohlc_btc_cc_pkl_file_name = f'{filename_stub}_ohlc_btc_cc.pkl'
+
+
+    if recompute_all:
+        client = bitmex.bitmex(test=False, config=None, api_key=public_key, api_secret=private_key)
+
+        #getting all the wallet information
+        #wallet_valo_histo = pd.DataFrame(client.User.User_getWalletHistory().result()[0])
+        actual_positions = pd.DataFrame(client.Position.Position_get().result()[0])
+
+        if isinstance(start_time,dt.datetime) and isinstance(end_time,dt.datetime):
+            start_time = start_time.strftime('%Y-%m-%dT%H:%M')
+            end_time = end_time.strftime('%Y-%m-%dT%H:%M')
+
+        ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+
+        ce = ce - dt.timedelta(hours=8)
+        trades = pd.DataFrame()
+        print('Accessing Trade History...')
+        while ce < te:
+            print(f'requesting trades between {ce} and {te}')
+            tmp = pd.DataFrame(client.Execution.Execution_getTradeHistory(startTime=ce, count=500).result()[0]).sort_values(
+                by='timestamp', ascending=True)
+            trades = trades.append(tmp)
+            time.sleep(4)
+            ce = tmp.timestamp.iloc[-1]
+        trades = trades[trades.timestamp <= te].copy()
+        trades.execComm = - trades.execComm  # logique comptable
+        trades = trades.sort_values(by='timestamp', ascending=True).drop_duplicates()
+        print('Done trades.')
+
+        # Get wallet history for a period of time
+        ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+        ce = ce - dt.timedelta(hours=8)
+        walletHist = pd.DataFrame()
+        print('Accessing wallet history...')
+        while ce < te:
+            print(f'requesting wallet history between {ce} and {te}')
+            tmp = pd.DataFrame(client.User.User_getWalletHistory(currency='XBt').result()[0]).sort_values(by='timestamp',
+                                                                                                          ascending=True)
+            walletHist = walletHist.append(tmp)
+            time.sleep(3.5)
+            ce = tmp.timestamp.iloc[-1]
+        walletHist = walletHist[walletHist.timestamp <= te].copy()
+        print('Wallet History retrieved.')
+
+        # Récupérer les OHLC hourly sur Cryptocompare depuis startdate
+        cc = CryptoCompare('')
+        ohlc_btc_cc = cc.get('OHLC', 'BTC', start_time, end_time, precision='hour')
+
+        if save_to_dropbox :
+            full_path = local_root_directory + trade_pkl_file_name
+            trades.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=trade_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + reportvalo_pkl_file_name
+            walletHist.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=reportvalo_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_btc_cc_pkl_file_name
+            ohlc_btc_cc.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_btc_cc_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + positions_pkl_file_name
+            actual_positions.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=positions_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+    else:
+        # trades=dbx.download_pkl(trade_pkl_file_name)
+        # walletHist=dbx.download_pkl(reportvalo_pkl_file_name)
+        # ohlc_btc_cc=dbx.download_pkl(ohlc_btc_cc_pkl_file_name)
+        # actual_positions=dbx.download_pkl(positions_pkl_file_name)
+
+        trades=pd.read_pickle(local_root_directory +trade_pkl_file_name)
+        walletHist=pd.read_pickle(local_root_directory +reportvalo_pkl_file_name)
+        ohlc_btc_cc=pd.read_pickle(local_root_directory +ohlc_btc_cc_pkl_file_name)
+        actual_positions=pd.read_pickle(local_root_directory +positions_pkl_file_name)
+    ##############################
+    ##############################
+    ##########
+    ########## computing positions
+    ##########
+    ##############################
+    ##############################
+    print(actual_positions.shape)
+    actual_positions.timestamp = actual_positions.timestamp.apply(lambda x: x.timestamp() * 1000)
+    actual_positions['timestamp'] = pd.to_datetime(actual_positions['timestamp'],unit='ms')
+    actual_positions['totalAsset']=actual_positions['currentQty']/actual_positions['lastPrice']
+    posColumns=[
+        'timestamp',
+        'symbol',
+        'lastPrice',
+        'totalAsset'
+    ]
+    actual_positions = actual_positions[posColumns].copy()
+    actualPositions = actual_positions.rename(columns={
+        'symbol':'asset',
+        'totalAsset':'amount',
+        'lastPrice':'price'
+    })
+
+    print('done')
+
+    ##############################
+    ##############################
+    ##########
+    ########## computing trades
+    ##########
+    ##############################
+    ##############################
+    trades = trades.sort_values(by='timestamp', ascending=True).drop_duplicates()
+    # trades.transactTime = trades.transactTime.apply(lambda x: x.timestamp() * 1000)
+    # trades['transactTime'] = pd.to_datetime(trades['transactTime'])
+    # trades = trades.drop(columns=['timestamp'])
+    # trades.to_excel('/Users/stefanduprey/Documents/My_SlippageFiles/b40lux_september_bitmex_raw_trades.xls',
+    #                 engine='xlsxwriter', index=False)
+    # raise Exception('mouteur foureur')
+    print('preprocessing trades')
+    bitmex_df = preprocess_bitmex_trades(trades)
+    print('preporcessing done')
+    print(bitmex_df.shape)
+
+
+    ##############################
+    ##############################
+    ##########
+    ########## computing valos
+    ##########
+    ##############################
+    ##############################
+    # ohlc_btc_cc
+    x = ohlc_btc_cc.copy()
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_btc_cc' for y in x.columns]
+    ohlc_btc_cc = x.copy()
+
+    walletHist = walletHist.set_index('timestamp')
+    walletHist.index = walletHist.index.map(lambda x: x.floor('60s'))
+    walletHist = walletHist[~walletHist.index.duplicated(keep='first')].sort_index(ascending=True)
+    walletHist.index = walletHist.index.tz_localize(None)
+    walletHist['walletBalance_xbt'] = walletHist.walletBalance / 100000000
+    walletHist = walletHist.join(ohlc_btc_cc['open_btc_cc'])
+    walletHist['open_btc_cc'] = walletHist['open_btc_cc'].bfill()
+    walletHist['walletBalance_usd'] = walletHist.walletBalance_xbt * walletHist.open_btc_cc
+
+
+
+    ########################################################################
+    ########################################################################
+    ################### gettingn the proper trades df
+    bitmex_df = bitmex_df.rename(columns={
+        'execType': 'Nature',
+        'symbol': 'Security Code',
+        'side': 'Oper',
+        'quantity': 'Amount',
+        'price': 'Price',
+        'total_fees_usd': 'Fees',
+        'transactTime': 'Date'
+    })
+
+    bitmex_df['Currency'] = 'USD'
+
+    trades_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Amount','Price', 'Fees']
+
+    bitmex_df = bitmex_df[trades_columns]
+    bitmex_df.loc[bitmex_df['Nature'] == 'Funding', 'Oper'] = 'Funding'
+    bitmex_df.loc[bitmex_df['Nature'] == 'Funding', 'Price'] = np.nan
+    bitmex_df.loc[bitmex_df['Nature'] == 'Funding', 'Amount'] = np.nan
+
+    if not by_pass_month:
+        bitmex_df['month'] = bitmex_df['Date'].dt.month
+        bitmex_df = bitmex_df[bitmex_df['month'] == me_month].copy()
+        bitmex_df = bitmex_df.drop(columns=['month'])
+
+
+    ############## getting the proper wallet Hist
+    filteredWallet = walletHist.copy()
+    filteredWallet['Date'] = filteredWallet.index
+    filteredWallet['month'] = filteredWallet['Date'].dt.month
+    filteredWallet['insideMonth'] = filteredWallet['month'] == me_month
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['insideMonth'].shift(-1)
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['shiftedInsideMonth'].fillna(True)
+    filteredWallet = filteredWallet[filteredWallet['shiftedInsideMonth']].copy()
+    filteredWallet = filteredWallet.drop(columns=['month', 'insideMonth', 'shiftedInsideMonth'])
+
+    filteredWallet = filteredWallet.iloc[[0, -1]]
+
+    if not by_pass_month:
+        minDate = min(filteredWallet['Date'])
+        maxDate = max(filteredWallet['Date'])
+    else:
+        minDate = min(walletHist.index)
+        maxDate = max(walletHist.index)
+
+    filtered_bitmex_df = bitmex_df[bitmex_df['Date'] >= minDate].copy()
+    filtered_bitmex_df = filtered_bitmex_df[filtered_bitmex_df['Date'] <= maxDate].copy()
+
+    walletHist_df = walletHist[walletHist.index >= minDate].copy()
+    walletHist_df = walletHist_df[walletHist_df.index <= maxDate].copy()
+
+    #######
+    print('adding the valo transaction to the fees')
+    proper_walletHist_df = walletHist_df.copy()
+    proper_walletHist_df['Date'] = proper_walletHist_df.index
+    proper_walletHist_df = proper_walletHist_df.sort_values(by = 'Date')
+    proper_walletHist_df['PnL'] = proper_walletHist_df['walletBalance_usd'] - proper_walletHist_df['walletBalance_usd'].shift(1)
+
+    proper_walletHist_df['total_fees'] = 0.
+
+    for i in range(1,len(proper_walletHist_df)):
+        previous_filtering_ts = proper_walletHist_df.iloc[i-1, proper_walletHist_df.columns.get_loc('Date')]
+        filtering_ts = proper_walletHist_df.iloc[i, proper_walletHist_df.columns.get_loc('Date')]
+
+        local_filtered_bitmex_df = filtered_bitmex_df[filtered_bitmex_df['Date'] >= previous_filtering_ts].copy()
+        local_filtered_bitmex_df = local_filtered_bitmex_df[local_filtered_bitmex_df['Date'] <= filtering_ts].copy()
+        total_fees = local_filtered_bitmex_df['Fees'].sum()
+        proper_walletHist_df.iloc[i, proper_walletHist_df.columns.get_loc('total_fees')] = total_fees
+
+    proper_walletHist_df['PnL'] = proper_walletHist_df['PnL'] - proper_walletHist_df['total_fees']
+    proper_walletHist_df = proper_walletHist_df[['Date', 'PnL']].copy()
+    proper_walletHist_df['Nature'] ='Valo'
+    filtered_bitmex_df = pd.concat([filtered_bitmex_df,proper_walletHist_df]).copy()
+    filtered_bitmex_df = filtered_bitmex_df.sort_values(by = 'Date')
+    valo_columns=[
+        'open_btc_cc',
+        'walletBalance_usd',
+        'Date'
+    ]
+    #assert(filtered_bitmex_df['Value'].sum() == walletHist_df['realizedPnl'].sum())
+    walletHist_df['Date'] = walletHist_df.index
+
+    filteredWallet = filteredWallet[valo_columns].copy()
+
+    filteredWallet = filteredWallet.rename(columns = {
+        'walletBalance_usd': 'wallet_balance_usdt',
+        'open_btc_cc':'btc_price'
+    })
+
+    walletHist_df = walletHist_df.rename(columns = {
+        'walletBalance_usd': 'wallet_balance_usdt',
+        'open_btc_cc':'btc_price'
+    })
+
+    valo_columns=[
+        'Date',
+        'btc_price',
+        'wallet_balance_usdt',
+    ]
+
+    b40_cols = ['Nature', 'Security Code', 'Currency', 'Description', 'Comment', 'Oper', 'Date', 'Value', 'Settlement',
+                'Amount', 'Price', 'Int. Rate', 'Meth', 'Excl', 'Last Coupon', 'Interest', 'Depo', 'Bank', 'Pmt Cur',
+                'Source	Investor', 'Fees', 'Bank', 'PnL']
+    for me_col in b40_cols:
+        if me_col not in filtered_bitmex_df.columns:
+            filtered_bitmex_df[me_col] = np.nan
+
+    filtered_bitmex_df = filtered_bitmex_df[b40_cols].copy()
+    filWal = filteredWallet[valo_columns].copy()
+    wal =  walletHist_df[valo_columns]
+
+    filtered_bitmex_df = filtered_bitmex_df.sort_values(by=['Date'])
+    filWal = filWal.sort_values(by=['Date'])
+    wal = wal.sort_values(by=['Date'])
+
+
+    return filtered_bitmex_df,filWal,wal,actual_positions
+
+
+def fetch_minute_data(ssj = 'XBT', public_key = None, private_key= None, start_time= None, end_time= None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './'):
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Bitmex'
+
+    ohlc_eth_bm_pkl_file_name = f'{filename_stub}_ohlc_eth_bm.pkl'
+    ohlc_btc_bm_pkl_file_name = f'{filename_stub}_ohlc_btc_bm.pkl'
+
+    if isinstance(start_time,dt.datetime) and isinstance(end_time,dt.datetime):
+        start_time = start_time.strftime('%Y-%m-%dT%H:%M')
+        end_time = end_time.strftime('%Y-%m-%dT%H:%M')
+    #
+    #
+    # # Récupérer les OHLC hourly sur Cryptocompare depuis startdate
+    # cc = CryptoCompare('')
+    # ohlc_btc_cc = cc.get('OHLC', 'BTC', start_time, end_time, precision='hour')
+    # ohlc_eth_cc = cc.get('OHLC', 'ETH', start_time, end_time, precision='hour')
+    # print("OHLC Cryptocompare retrieved.")
+
+    # Récupérer les OHLC hourly sur BM depuis startdatex
+    bm = BitMEX(public_key, private_key)
+    ohlc_btc_bm = bm.get('OHLC', currency=ssj, startdate=start_time, enddate=end_time, precision='1m')
+    return ohlc_btc_bm
+
+
+
 def fetch_all_necessary_data(public_key = None, private_key= None, start_time= None, end_time= None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './'):
 
     dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
@@ -544,7 +896,6 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
         start_time = start_time.strftime('%Y-%m-%dT%H:%M')
         end_time = end_time.strftime('%Y-%m-%dT%H:%M')
 
-
     ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
 
     ce = ce - dt.timedelta(hours=8)
@@ -615,7 +966,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
         dbx.upload(fullname=full_path, folder='', subfolder='', name=walletHist_pkl_file_name, overwrite=True)
         os.remove(full_path)
 
-        full_path = './' + ohlc_btc_cc_pkl_file_name
+        full_path = local_root_directory + ohlc_btc_cc_pkl_file_name
         ohlc_btc_cc.to_pickle(full_path)
         print(f'uploading to dropbox {full_path}')
         dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_btc_cc_pkl_file_name, overwrite=True)
@@ -738,7 +1089,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
               'trading_fees_eth_usd', 'funding_fees_eth_usd', 'slippage_eth_price_cc', 'slippage_eth_cash_cc',
               'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_cc', 'slippage_btc_cash_cc',
               'trading_fees_eth_usd', 'funding_fees_eth_usd', 'slippage_eth_price_bm', 'slippage_eth_cash_bm',
-              'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_bm', 'slippage_btc_cash_bm']]
+              'trading_fees_xbt_usd', 'funding_fees_xbt_usd', 'slippage_btc_price_bm', 'slippage_btc_cash_bm','total_fees_usd']]
 
     x = x.rename_axis('timestamp')
     x.columns = ['quantity_traded_eth', 'avg_traded_price_eth', 'amount_traded_usd_eth', 'n_trades_eth',
@@ -751,7 +1102,7 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
                  'trading_fees_usd_eth', 'funding_fees_usd_eth', 'slippage_eth_price_cc', 'slippage_eth_cash_cc',
                  'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_cc', 'slippage_btc_cash_cc',
                  'trading_fees_usd_eth', 'funding_fees_usd_eth', 'slippage_eth_price_bm', 'slippage_eth_cash_bm',
-                 'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_bm', 'slippage_btc_cash_bm']
+                 'trading_fees_usd_btc', 'funding_fees_usd_btc', 'slippage_btc_price_bm', 'slippage_btc_cash_bm','total_fees_usd']
 
     x = x.fillna(0).round(decimals=5)
 
